from django.contrib import admin
from .models import CookieConsentLog, CookieConsentSettings, CookieScript
from .app_config import DJANGO_VERSION_LESS_THAN_2

if DJANGO_VERSION_LESS_THAN_2:
    from django.utils.translation import ugettext_lazy as _
else:
    from django.utils.translation import gettext_lazy as _


@admin.register(CookieScript)
class CookieScriptAdmin(admin.ModelAdmin):
    list_display = ('cookie_name', 'cookie_type', 'cookie_script')


@admin.register(CookieConsentSettings)
class CookieConsentSettingsAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': (),
            'description': "<h2>Attenzione: ogni volta che questa pagna viene salvata, aumenta il numero di revisioni del banner, e all\'utente verrà rimostrato il banner per acconsentire alla nuova versione dei cookie</h2>"
        }),
        ("Banner", {
            'fields': ('title_banner', 'description_banner', 'layout_banner', 'position_y_banner', 'position_x_banner')
        }),
        ("Modal", {
            'fields': ('title_modal', 'description_modal', 'layout_modal', 'position_modal')
        }),
        ("Descrizione cookie", {
            'fields': ('description_technical', 'description_analytics', 'description_marketing')
        }),
        ("Altro", {
            'fields': ('description_information', )
        }),
        (None, {
            'fields': ('banner_is_active',),
            'description': "<h2>Modalità di utilizzo: <ul style='margin-left:12px;'><li style='list-style-type:square;'>Attivando il banner hai la possibilità di gestire il consenso di tutti i cookie;</li><li style='list-style-type:square;'>Se invece disattivi il banner, verranno mostrati solo i cookie tecnici/necessari, mostrando all'utente solo il banner informativo.</li></ul></h2>"
        }),
    )
    # list_display = ()
    # search_fields = []

    def has_add_permission(self, request):
        if DJANGO_VERSION_LESS_THAN_2:
            return False if self.model.objects.count() > 0 else super(CookieConsentSettingsAdmin, self).has_add_permission(request)
        else:
            return False if self.model.objects.count() > 0 else super().has_add_permission(request)

    def has_delete_permission(self, request, obj=None):
        return False


@admin.register(CookieConsentLog)
class CookieConsentLogAdmin(admin.ModelAdmin):
    list_display = ('consent_date_time', 'consent_token', 'consent_url', 'consent_cookie')
    search_fields = ['consent_token', 'consent_date_time']

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False



from django.db import models
from .app_config import COOKIEBANNER_IS_WAGTAIL_PROJECT, DJANGO_VERSION_LESS_THAN_2

if DJANGO_VERSION_LESS_THAN_2:
    from django.utils.translation import ugettext_lazy as _
else:
    from django.utils.translation import gettext_lazy as _

if COOKIEBANNER_IS_WAGTAIL_PROJECT:
    from wagtail.admin.edit_handlers import FieldPanel, RichTextField, MultiFieldPanel, HelpPanel


COOKIE_TYPE_CHOICE = (
    (0, _('Tecnici/Necessari')),
    (1, _('Statistiche')),
    (2, _('Marketing')),
)


class CookieScript(models.Model):
    cookie_name = models.CharField(_("Nome"), max_length=200)
    cookie_type = models.IntegerField(_("Tipologia"), choices=COOKIE_TYPE_CHOICE, default=1)
    cookie_script = models.TextField(_("Script JS"))

    def __str__(self):
        return self.cookie_name

    class Meta:
        verbose_name_plural = "Cookie Scripts"
        ordering = ['cookie_type', ]


class CookieConsentLog(models.Model):
    consent_token = models.CharField(_("Chiave consenso"), max_length=50, unique=True)
    consent_url = models.URLField(_("Consenso Cookie Url"), max_length=200)
    consent_date_time = models.DateTimeField(_("Data e Ora"), auto_now=False, auto_now_add=True, editable=False)
    consent_user_agent = models.TextField(_("User Agent"))
    consent_anonymize_ip = models.CharField(_("IP Anonymize"), max_length=20)
    consent_cookie = models.TextField(_("Lista cookie accettati"))

    def __str__(self):
        return self.consent_token

    class Meta:
        verbose_name_plural = "Cookie Log"
        ordering = ['-consent_date_time', ]


class CookieConsentSettings(models.Model):
    title_banner = models.CharField(_("Titolo banner"), max_length=255, blank=True)
    title_modal = models.CharField(_("Titolo modal"), max_length=255, blank=True)

    if COOKIEBANNER_IS_WAGTAIL_PROJECT:
        description_banner = RichTextField(_("Descrizione banner"), features=['link'], blank=True)
        description_modal = RichTextField(_("Descrizione modal"), features=['link'], blank=True)
        description_technical = RichTextField(_("Descrizione cookie tecnici"), features=['bold', 'link'], blank=True)
        description_analytics = RichTextField(_("Descrizione cookie statistici"), features=['bold', 'link'], blank=True)
        description_marketing = RichTextField(_("Descrizione cookie marketing"), features=['bold', 'link'], blank=True)
        description_information = RichTextField(_("Maggiori informazioni"), features=['bold', 'link'], blank=True)
    else:
        description_banner = models.TextField(_("Descrizione banner"), blank=True)
        description_modal = models.TextField(_("Descrizione modal"), blank=True)
        description_technical = models.TextField(_("Descrizione  cookie tecnici"), blank=True)
        description_analytics = models.TextField(_("Descrizione  cookie statistici"), blank=True)
        description_marketing = models.TextField(_("Descrizione cookie marketing"), blank=True)
        description_information = models.TextField(_("Maggiori informazioni"), blank=True)

    layout_banner = models.CharField(_("Layout banner"), max_length=255, choices=(('box', 'Box'), ('cloud', 'Cloud'), ('bar', 'Bar')), default='box')
    position_y_banner = models.CharField(_("Posizione Y banner"), max_length=255, choices=(('top', 'Top'), ('middle', 'Middle'), ('bottom', 'Bottom')), default='bottom')
    position_x_banner = models.CharField(_("Posizione X banner"), max_length=255, choices=(('left', 'Left'), ('center', 'Center'), ('right', 'Right')), default='left')
    layout_modal = models.CharField(_("Layout modal"), max_length=255, choices=(('box', 'Box'), ('bar', 'Bar')), default='box')
    position_modal = models.CharField(_("Posizione modal"), max_length=255, choices=(('left', 'Left'), ('right', 'Right')), default='left')

    banner_is_active = models.BooleanField(_("Attivare il banner?"), default=True)
    revision_banner = models.IntegerField(default=0)

    if COOKIEBANNER_IS_WAGTAIL_PROJECT:
        panels = [
            HelpPanel(content='<div class="help-block help-info"><p>Attenzione: ogni volta che questa pagna viene salvata, aumenta il numero di revisioni del banner, e all\'utente verrà rimostrato il banner per acconsentire alla nuova versione dei cookie</p></div>'),
            MultiFieldPanel([
                FieldPanel("title_banner"),
                FieldPanel("description_banner"),
                FieldPanel("layout_banner"),
                FieldPanel("position_y_banner"),
                FieldPanel("position_x_banner"),
            ], heading="Banner"),
            MultiFieldPanel([
                FieldPanel("title_modal"),
                FieldPanel("description_modal"),
                FieldPanel("layout_modal"),
                FieldPanel("position_modal"),
            ], heading="Modal"),
            MultiFieldPanel([
                FieldPanel("description_technical"),
                FieldPanel("description_analytics"),
                FieldPanel("description_marketing"),
            ], heading="Descrizione cookie"),
            FieldPanel('description_information'),
            MultiFieldPanel([
                HelpPanel(content="<ul style='margin-left:12px;margin-top:-40px;margin-bottom:-24px;'><li style='list-style-type:square;'>Attivando il banner hai la possibilità di gestire il consenso di tutti i cookie;</li><li style='list-style-type:square;'>Se invece disattivi il banner, verranno mostrati solo i cookie tecnici/necessari, mostrando all'utente solo il banner informativo.</li></ul>"),
                FieldPanel("banner_is_active"),
            ], heading="Modalità di utilizzo"),
        ]

    def save(self, *args, **kwargs):
        self.revision_banner = self.revision_banner + 1
        if DJANGO_VERSION_LESS_THAN_2:
            super(CookieConsentSettings, self).save(*args, **kwargs)
        else:
            super().save(*args, **kwargs)

    def __str__(self):
        return 'Settings'

    class Meta:
        verbose_name_plural = "Cookie settings"



import json
from django import template
from ..models import CookieScript, CookieConsentSettings
import re


register = template.Library()


@register.simple_tag(takes_context=True)
def gdpr_cookie_banner(context):
    t = template.loader.get_template('cookiebanner.html')
    data = {}
    data['cookie_scripts'] = CookieScript.objects.all()
    data['cookie_settings'] = CookieConsentSettings.objects.all().first()
    return t.render(data, request=context['request'])


@register.filter()
def regex_script(script, script_type):
    script = re.sub(r'(<script)', r'\1 type="text/plain" data-cookiecategory="' + script_type + r'"', script)
    return script



from django.views import View
from django.http.response import JsonResponse
from django.http import HttpResponse

import string
import random
import datetime
import csv

from .models import CookieConsentLog


# Create your views here.
class CookieCreateView(View):
    def post(self, request, **kwargs):
        if request.is_ajax():
            cookie_preferences = request.POST.get('user_preferences')
            cookie_from_url = request.POST.get('request_url')

            # Anonymize the IP Address by hidden last group
            consent_anonymize_ip = None
            if request.META.get('HTTP_X_FORWARDED_FOR'):
                consent_anonymize_ip = request.META.get('HTTP_X_FORWARDED_FOR').split(',')[0]
            else:
                consent_anonymize_ip = request.META.get('REMOTE_ADDR')
            ca_ip = consent_anonymize_ip.split('.')
            consent_anonymize_ip = '{one}.{two}.{three}.0'.format(one=ca_ip[0], two=ca_ip[1], three=ca_ip[2])

            cookie_model_obj = CookieConsentLog()
            cookie_model_obj.consent_token = ''.join(random.choice(string.ascii_letters + string.digits) for i in range(35)) + '=' + str(datetime.datetime.now().strftime("%Y%m%d%H%M%S"))
            cookie_model_obj.consent_url = cookie_from_url
            cookie_model_obj.consent_user_agent = request.META['HTTP_USER_AGENT']
            cookie_model_obj.consent_anonymize_ip = consent_anonymize_ip
            cookie_model_obj.consent_cookie = cookie_preferences
            cookie_model_obj.save()

            data = {'consent_token': cookie_model_obj.consent_token}

            response = JsonResponse(data, safe=False)
            return response


class DjangoExportView(View):
    def get(self, request, **kwargs):
        model_obj = CookieConsentLog.objects.all().first()._meta.fields
        field_names = [field.name for field in model_obj]

        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename=Cookie_Consent_Log-{date_time}.csv'.format(date_time=datetime.datetime.now())

        writer = csv.writer(response)
        writer.writerow(field_names)
        for obj in CookieConsentLog.objects.all():
            row = writer.writerow([getattr(obj, field) for field in field_names])

        return response


from .app_config import COOKIEBANNER_IS_WAGTAIL_PROJECT


if COOKIEBANNER_IS_WAGTAIL_PROJECT:
    from wagtail.contrib.modeladmin.options import ModelAdmin, ModelAdminGroup, modeladmin_register
    from wagtail.contrib.modeladmin.helpers import PermissionHelper
    from .models import CookieConsentLog, CookieScript, CookieConsentSettings
    import datetime

    class CookieConsentLogValidationPermissionHelper(PermissionHelper):
        def user_can_list(self, user):
            return True

        def user_can_create(self, user):
            return False

        def user_can_edit_obj(self, user, obj):
            return False

        def user_can_delete_obj(self, user, obj):
            return False

    class CookieConsentSettingsValidationPermissionHelper(PermissionHelper):
        def user_can_list(self, user):
            return True

        def user_can_create(self, user):
            return False if self.model.objects.count() > 0 else True

        def user_can_delete_obj(self, user, obj):
            return False

    class CookieConsentLogAdmin(ModelAdmin):
        model = CookieConsentLog
        permission_helper_class = CookieConsentLogValidationPermissionHelper
        inspect_view_enabled = True
        menu_label = "Cookie log"
        menu_icon = "pick"
        exclude_from_explorer = False
        list_export = ('consent_date_time', 'consent_token', 'consent_url', 'consent_user_agent', 'consent_anonymize_ip')
        export_filename = 'Cookie_Consent_Log_' + str(datetime.datetime.now())
        list_display = ('consent_date_time', 'consent_token', 'consent_url', 'consent_anonymize_ip')
        search_fields = ['consent_token', 'consent_date_time']

    class CookieScriptAdmin(ModelAdmin):
        model = CookieScript
        list_display = ('cookie_name', 'cookie_type', 'cookie_script')

    class CookieConsentSettingsAdmin(ModelAdmin):
        model = CookieConsentSettings
        permission_helper_class = CookieConsentSettingsValidationPermissionHelper
        menu_label = "Cookie settings"
        menu_icon = "cog"
        # list_display = ('campo',)

    class GdprSolutionGroup(ModelAdminGroup):
        menu_label = 'GDPR Solution'
        menu_icon = 'pick'
        menu_order = 1000
        items = (CookieScriptAdmin, CookieConsentLogAdmin, CookieConsentSettingsAdmin)

    modeladmin_register(GdprSolutionGroup)



import django
from pkg_resources import parse_version

try:
    from wagtail import __version__ as WAGTAIL_VERSION
except ImportError:
    WAGTAIL_VERSION = False


DJANGO_VERSION_LESS_THAN_2 = False
if parse_version(django.get_version()) < parse_version('2.0.0'):
    DJANGO_VERSION_LESS_THAN_2 = True


COOKIEBANNER_IS_WAGTAIL_PROJECT = False
if WAGTAIL_VERSION and parse_version(WAGTAIL_VERSION) >= parse_version('1.13.4'):
    COOKIEBANNER_IS_WAGTAIL_PROJECT = True



from .views import CookieCreateView, DjangoExportView
from .app_config import DJANGO_VERSION_LESS_THAN_2


if DJANGO_VERSION_LESS_THAN_2:
    from django.conf.urls import url
else:
    from django.urls import path


if DJANGO_VERSION_LESS_THAN_2:
    urlpatterns = [
        url(r'^create-cookie/', CookieCreateView.as_view(), name='create_cookie_view'),
        url(r'^export-django/', DjangoExportView.as_view(), name='export_django'),
    ]
else:
    urlpatterns = [
        path('create-cookie/', CookieCreateView.as_view(), name='create_cookie_view'),
        path('export-django/', DjangoExportView.as_view(), name='export_django'),
    ]



#!/usr/bin/env python

from os import path
from setuptools import find_packages, setup
from io import open

from gdpr_solution import __version__

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="django-gdpr-solution",
    version=__version__,
    description="Add GDPR compliant cookie consent to your website.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Lorenzo",
    # author_email="email@email.com",
    url="https://github.com/lorenzomonte-it/django-gdpr-solution",
    project_urls={
        'Homepage': 'https://github.com/lorenzomonte-it/django-gdpr-solution',
    },
    packages=find_packages(),
    include_package_data=True,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Framework :: Django",
        "Framework :: Wagtail"
    ],
    install_requires=["Django>=1.11.0"],
    zip_safe=False,
)
